
/* ++++++++++

   FILE:  AttributeView.cpp
   NAME:  joseph jones

   Copyright (c) 1996 by joseph jones.  All Rights Reserved.

+++++ */

#pragma once

#include <Directory.h>
#include <Window.h>
#include <Rect.h>
#include <View.h>
#include <ScrollView.h>
#include <Alert.h>
#include <AppDefs.h>
#include <Message.h>
#include <Errors.h>
#include <Path.h>
#include <image.h>

#include <Debug.h>

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "JLabelView.h"

#define MIME_TYPE 'MIMS'

extern char** environ;

class TStatusView : public BOutlineListView {
public:
                   TStatusView(BRect);

};

class TStatusWindow : public BWindow {
	BEntry*			pEntry;
	TStatusView*	m_pView;
	BScrollView*	m_psview;
	
public:
    TStatusWindow(BRect, const char*);
	
	~TStatusWindow();
	
	void SetEntry(BEntry* pentry);
	
private:

	void AddDataItem( BListItem* pPItem, const BNode& node, const attr_info& info, const char* attrName);

};

#pragma export on
void process_refs(entry_ref dir_ref, BMessage* msg, void*);
#pragma export reset

void process_refs(entry_ref dir_ref, BMessage* msg, void*)
{

    BAlert*         alert;
    TStatusWindow*  wind = NULL;
    ulong           type;
    entry_ref       ref;
    int32           result;
    long            count;
    long            i;

    // this add-on requires a selection
    msg->GetInfo("refs", &type, &count);
    
    if (count == 0) 
    {
        alert = new BAlert("", "You must have a selection to make a tar archive.", "Cancel");
        alert->Go();
        return;
    }


	BRect win_rect(100, 100, 400, 300);
	
	//Get Entry Name for Tab Title
	//ptentry->GetName(name);
	
    wind = new TStatusWindow(win_rect, "Attribute Browser");

    //printf("Is this it?\n");
    
    for (i = 0; i < count; i++) 
    {

        if (msg->FindRef("refs", i, &ref) == B_NO_ERROR)
        {
            /*
            	Create a new window for each entry
            	and pass the new window the specific entry
            	we get here.
            */
            
            BEntry*		ptentry = NULL;
            
            ptentry = new BEntry(&ref);
            
            if (wind)
            {
            	wind->SetEntry(ptentry);
            }

           	delete ptentry;
		}            
        else 
        {
            alert = new BAlert("", "There was an error finding one of the items you chose to archive.", "Cancel");
            alert->Go();
            count = i;
			return;
        }
    }

   	wind->Show();
	
	thread_id 	tid;
	long		t_exit_val;
		
	tid = wind->Thread();
	
	wait_for_thread(tid, &t_exit_val);

}

// ---------------------------------------------------------------

TStatusView::TStatusView(BRect r)
            :BOutlineListView(	r, 
            			"", 
            			B_SINGLE_SELECTION_LIST, 
            			B_FOLLOW_ALL, 
            			B_FRAME_EVENTS | B_WILL_DRAW | B_NAVIGABLE)
{
           	printf("Window Constructed\n");
}

// ---------------------------------------------------------------


TStatusWindow::TStatusWindow(BRect r, const char *title)
              :pEntry(NULL),
              BWindow(r, title, B_TITLED_WINDOW, 0)
{
    //Create Label
    BRect label_rect(0, 0, 300, 19);
    
    label_rect.OffsetTo(B_ORIGIN);
    
    JLabelView* pLabel = new JLabelView(label_rect, title, (B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP), B_WILL_DRAW | B_FRAME_EVENTS);
		
	//pLabel->SetText(path.Path());
	pLabel->SetText("Attribute Browser");
	pLabel->SetViewColor(200, 200, 200);
	
	AddChild(pLabel);

    //Create Attribute List View
    BRect   status_wind_rect(1, 21, 299, 199);
	
    status_wind_rect.right -= B_V_SCROLL_BAR_WIDTH;
    status_wind_rect.bottom -= B_H_SCROLL_BAR_HEIGHT;
    
    //status_wind_rect.OffsetTo(B_ORIGIN);
    m_pView = new TStatusView(status_wind_rect);
    
    //m_pView->SetViewColor(200, 200, 200);

	m_psview = new BScrollView("", m_pView, B_FOLLOW_ALL, B_WILL_DRAW, true, true);
    
    m_pView->TargetedByScrollView(m_psview);
    AddChild(m_psview);

}

TStatusWindow::~TStatusWindow()
{
	if (pEntry)
	{
		delete pEntry;
		pEntry = NULL;
	}
}

void TStatusWindow::SetEntry(BEntry* ptentry)
{
	Lock();
	
	if (ptentry)
	{
	    
	    
	    //Add file path to ListView
	    BPath	path;
	    
	    ptentry->GetPath(&path);

		//Loop through attributes
		char 				attrName[100];
		char				type[5];
		char				listLine[300];
		
		struct attr_info	info;
		
		BNode node(ptentry);		
		
		ptentry->GetName(listLine);
		
		BStringItem* pFileItem = new BStringItem(listLine);
		
		m_pView->AddItem(pFileItem);

		while(node.GetNextAttrName(attrName) == B_NO_ERROR)
		{
			char* pname = strdup(attrName);
		
			node.GetAttrInfo(attrName, &info);

			sprintf(listLine, "Attribute Name : %s", attrName);
			BStringItem* item = new BStringItem(listLine);

			//Add Item
			m_pView->AddUnder(item, pFileItem);

			//Add Sub Items
			AddDataItem(item, node, info, attrName);

			sprintf(listLine,"Size : %Ld", info.size );

			BStringItem* subitem = new BStringItem(listLine);
			m_pView->AddUnder(subitem, item);

			strncpy(type, (char*)&(info.type), 4);
			type[4] = '\0';
			
			sprintf(listLine, "Type : %s", type);
			subitem = new BStringItem(listLine);
			m_pView->AddUnder(subitem, item);

			//m_pView->Collapse(item);
		}
		
		//m_pView->Collapse(pFileItem);			

	}
	
	Unlock();
}

void TStatusWindow::AddDataItem( BListItem* pPItem, const BNode& node, const attr_info& info, const char* attrName)
{

	void*	pData = NULL;
	
	//Only add if a size exists
	if (info.size > 0)
	{
		//Allocate enough data to hold attribute
		pData = malloc(info.size);
		
		if (pData)
		{
			
			//ReadAttribute
			if (node.ReadAttr(attrName, info.type, 0, pData, info.size))
			{
				switch(info.type)
				{
				 	case B_STRING_TYPE:
				 	case B_ASCII_TYPE:
				 	case MIME_TYPE:
				 	case B_MIME_TYPE:
				 	{
						char* stringBuf = (char*)malloc(info.size+1);
						char* listbuf = (char*)malloc((info.size+1)+8);
						
						if (stringBuf != NULL)
						{
							stringBuf[info.size] = '\0';
							strncpy(stringBuf, (char*)pData, info.size);
							
							sprintf(listbuf, "Value : %s", stringBuf);
							
							BStringItem* pItem = new BStringItem(listbuf);
							m_pView->AddUnder(pItem, pPItem);
							
							free(stringBuf);
							free(listbuf);
						}
						else 
						{
							char defString[] = "Value : <Unknown>";
							BStringItem* pItem = new BStringItem(defString);
							m_pView->AddUnder(pItem, pPItem);
						}
					}
					break;
					
					case B_RECT_TYPE:
					{
						BRect* pRect = (BRect*)pData;
						
						char listBuf[30];
						
						sprintf(listBuf, "Value : (%-9.2f, %-9.2f)x(%-9.2f, %-9.2f)",pRect->left,pRect->top,pRect->right,pRect->bottom);

						BStringItem* pItem = new BStringItem(listBuf);
						m_pView->AddUnder(pItem, pPItem);
					}
					break;
					
					case B_CHAR_TYPE:
					{
						char* stringBuf = (char*)malloc(sizeof(char) + 1);
						char* listbuf = (char*)malloc(sizeof(char)+8);

						if (stringBuf != NULL)
						{
							stringBuf[sizeof(char) + 1] =     '\0';
							stringBuf[0] = ((char*)pData)[0];
							
							sprintf(listbuf, "Value : %s", stringBuf);
							
							BStringItem* pItem = new BStringItem(listbuf);
							m_pView->AddUnder(pItem, pPItem);
							
							free(stringBuf);
							free(listbuf);
						}
						else 
						{
							char defString[] = "Value : <Unknown>";
							BStringItem* pItem = new BStringItem(defString);
							m_pView->AddUnder(pItem, pPItem);
						}
					}
					break;
					
					case B_DOUBLE_TYPE:
					{
						char dblString[18];
						sprintf(dblString, "Value : %f", *(double*)pData);
						BStringItem* pItem = new BStringItem(dblString);
						m_pView->AddUnder(pItem, pPItem);
						
					}
					break;

					default:
					{
						char defString[] = "Value : <Unknown>";
						BStringItem* pItem = new BStringItem(defString);
						m_pView->AddUnder(pItem, pPItem);
					}
					break;
				};
				
			}
			
			free(pData);
		}
	}
	
}
